!+
! Module: esources_mod
!
! Authors:  Ian Stewart - SSC/LUX
!          
! Description: Selects EPIC sources for creation of spectra and time series.
! 
! Comments: 
! 
! $Id: module.f90,v 1.6 1998/05/27 13:45:39 gvacanti Exp $
!- 
module esources_mod

  use dal
  use param  
  use errorHandling

  use dal_aux

  implicit none 

  private 

  type :: instWeightType
    integer(int16) :: idInst
    real(single)   :: weight
  end type instWeightType

  ! Command-line parameters:
  character(256), allocatable ::&
    eventSetNames(:)
  character(256) ::&
    srcmatchSourceSetTabName='',&
    outputSourceSetName=''
  real(single) ::&
    minEpicCounts=0.0,&
    minDetml=0.0,&
    minMaskFrac=0.0
  integer ::&
    maxNumHexSrcChars=0 ! [1:7]
  type(instWeightType), allocatable ::&
    weights(:)
  logical ::&
    setFlagsInInputList=.false.

  ! Other globals:
  character(256) :: srcmatchSourceSetName
  character(20)  :: srcmatchSourceTabName

  type :: instSrcDataType
    logical      :: sctsIsNull
    real(single) :: scts, detml, maskFrac
    real(double) :: ontime
  end type instSrcDataType

  type :: srcDataType
    real(single)          :: scts, binWidth
    logical               :: sctsIsNull
    type(instSrcDataType) :: PN, M1, M2
    character(7)          :: srcNumHex 
    integer(int32)        :: bitFlags, srcNum, mlIdSrc
    real(double)          :: ra, dec, raCorr, decCorr
  end type srcDataType

  integer, parameter :: pnFlagBit = 0, m1FlagBit = 1, m2FlagBit = 2

  public esources

contains 

  !---------------------------------------------------------------------
  subroutine esources
  !---------------------------------------------------------------------

    implicit none

    real(single), parameter :: pnMinSctsPerBin = 18.0,&
                               m1MinSctsPerBin =  5.0,&
                               m2MinSctsPerBin =  5.0

    integer :: maxSrcNum, i, numSources
    type(srcDataType), pointer :: srcData(:)
    real(single) :: usedWeight, binWidthReal, totalWeight
    real(double) :: startTime, stopTime

!logical :: testFlag = .false.

    !-------------------------------------------------------------------
    ! Start

    call readParameters

    maxSrcNum = 1 ! default
    do i = 1, maxNumHexSrcChars
      maxSrcNum = maxSrcNum * 16
    end do
    maxSrcNum = maxSrcNum - 1

    totalWeight = sum(weights%weight)

    call readSourceListData(srcData)
    numSources = size(srcData)

    do i = 1, numSources
      ! Compute total epic counts:

      usedWeight = 0.0

      if (.not.srcData(i)%PN%sctsIsNull) then
        srcData(i)%scts = srcData(i)%scts + srcData(i)%PN%scts
        usedWeight      = usedWeight      + weights(1)%weight
      end if

      if (.not.srcData(i)%M1%sctsIsNull) then
        srcData(i)%scts = srcData(i)%scts + srcData(i)%M1%scts
        usedWeight      = usedWeight      + weights(2)%weight
      end if

      if (.not.srcData(i)%M2%sctsIsNull) then
        srcData(i)%scts = srcData(i)%scts + srcData(i)%M2%scts
        usedWeight      = usedWeight      + weights(3)%weight
      end if

      if (usedWeight == 0.0) then
        srcData(i)%sctsIsNull = .true.
      else
        srcData(i)%scts = srcData(i)%scts * totalWeight / usedWeight
      end if
      ! Note that this formula is arithmetically identical to that used
      ! in ACS's script, except for the case in which SCTS is null only
      ! in one of the mos cameras. In this aberrant case the results
      ! of the two methods are equal on average, but the uncertainties
      ! are different.


      if (srcData(i)%scts >= minEpicCounts) then ! default 500
        if (  srcData(i)%PN%detml    >  minDetml&           ! default 15
        .and. srcData(i)%PN%maskFrac >= minMaskFrac)&       ! default 0.5
          srcData(i)%bitFlags = ibset(srcData(i)%bitFlags, pnFlagBit)

        if (  srcData(i)%M1%detml    >  minDetml&           ! default 15
        .and. srcData(i)%M1%maskFrac >= minMaskFrac)&       ! default 0.5
          srcData(i)%bitFlags = ibset(srcData(i)%bitFlags, m1FlagBit)

        if (  srcData(i)%M2%detml    >  minDetml&           ! default 15
        .and. srcData(i)%M2%maskFrac >= minMaskFrac)&       ! default 0.5
          srcData(i)%bitFlags = ibset(srcData(i)%bitFlags, m2FlagBit)
      end if


      ! Calculate bin width for each source (should give identical results
      ! to ACS):

      if (     .not.srcData(i)%PN%sctsIsNull .and. srcData(i)%PN%scts > 0.0)&
      then
        binWidthReal = floor(pnMinSctsPerBin * srcData(i)%PN%ontime&
          / srcData(i)%PN%scts)

      else if (.not.srcData(i)%M1%sctsIsNull .and. srcData(i)%M1%scts > 0.0)&
      then
        binWidthReal = floor(m1MinSctsPerBin * srcData(i)%M1%ontime&
          / srcData(i)%M1%scts)

      else if (.not.srcData(i)%M2%sctsIsNull .and. srcData(i)%M2%scts > 0.0)&
      then
        binWidthReal = floor(m2MinSctsPerBin * srcData(i)%M2%ontime&
          / srcData(i)%M2%scts)

      else
        call warning('bug0', "This shouldn't happen. Please contact &
          &the code developer.")
        srcData(i)%bitFlags = 0
      end if

      srcData(i)%binWidth = 10.0d0 * (1 + floor(binWidthReal/10.0))
      ! NOTE that this formula implies that there will always be LESS than
      ! the desired counts per bin.

      if (srcData(i)%binWidth <= 0.0) then
        call warning('binWidthTooSmall', "")
        srcData(i)%bitFlags = 0
      end if

      srcData(i)%srcNumHex = intToHex(srcData(i)%srcNum)

      if (srcData(i)%bitFlags .ne. 0 .and. srcData(i)%srcNum > maxSrcNum) then
        call warning('srcNumTooLarge', "")
        srcData(i)%bitFlags = 0
      end if
    end do

    call getStartAndStopTimes(startTime, stopTime)

    call writeSourceListData(srcData, startTime, stopTime)

    if (setFlagsInInputList .and. any(srcData%bitFlags > 0))&
      call setFlags((srcData%bitFlags > 0))

    deallocate(srcData)

  end subroutine esources

  !---------------------------------------------------------------------
  subroutine readParameters
  !---------------------------------------------------------------------

    implicit none

    integer :: numEventLists, i, numWeights

    !-------------------------------------------------------------------
    ! Start

    numEventLists = parameterCount('eventsets')
if (numEventLists <= 0) call error('noEventListsSupplied',"")
    allocate(eventSetNames(numEventLists))
    do i = 1, numEventLists
      eventSetNames(i) = stringParameter('eventsets',i-1)
    end do

    srcmatchSourceSetTabName = stringParameter('srctab')
    outputSourceSetName      = stringParameter('outsrcset')
    minEpicCounts            = realParameter(  'mintotalcts')
    minDetml                 = realParameter(  'mindetml')
    minMaskFrac              = realParameter(  'minmaskfrac')
    maxNumHexSrcChars        = intParameter(   'numhexchar') ! [1:7]

    numWeights = parameterCount('instweights')
if (numWeights .ne. 3) call error('badNumWeights',"")
    allocate(weights(numWeights))
    do i = 1, numWeights
      weights(i)%idInst = i
      weights(i)%weight = realParameter('instweights', i-1)
    end do

    setFlagsInInputList = booleanParameter('setflags')

  end subroutine readParameters

  !---------------------------------------------------------------------
  subroutine readSourceListData(srcData)
  !---------------------------------------------------------------------
  ! Note: the calling routine should deallocate the pointer srcData after use.

    implicit none

    type(srcDataType), pointer :: srcData(:)

!    character(256) :: setName
!    character(20) :: tabName
    type(DataSetT) :: set
    type(TableT) :: tab
    integer :: numSources, row
    type(ColumnT) ::&
      pnSctsCol, m1SctsCol, m2SctsCol, pnOntimeCol, m1OntimeCol, m2OntimeCol

    !-------------------------------------------------------------------
    ! Start

    call splitSetTabName(srcmatchSourceSetTabName, srcmatchSourceSetName&
      , srcmatchSourceTabName) ! in ssclib/dal_aux.

    set = dataSet(srcmatchSourceSetName, READ)
    tab = table(set, srcmatchSourceTabName)
    numSources = numberOfRows(tab)
    allocate(srcData(numSources))

    if (numSources > 0) then
      call initializeSrcDataType(srcData)

      ! The call readColDataToFixed is in ssclib/dal_aux.
      call readColDataToFixed(tab, 'SRC_NUM',  srcData%srcNum)
      call readColDataToFixed(tab, 'RA',       srcData%ra)
      call readColDataToFixed(tab, 'DEC',      srcData%dec)
      call readColDataToFixed(tab, 'RA_CORR',  srcData%raCorr)
      call readColDataToFixed(tab, 'DEC_CORR', srcData%decCorr)

      pnSctsCol = column(tab, 'PN_CTS')
      m1SctsCol = column(tab, 'M1_CTS')
      m2SctsCol = column(tab, 'M2_CTS')
      pnOntimeCol = column(tab, 'PN_ONTIME')
      m1OntimeCol = column(tab, 'M1_ONTIME')
      m2OntimeCol = column(tab, 'M2_ONTIME')

      do row = 0, numSources-1
        if (isNull(pnSctsCol, row) .or. isNull(pnOntimeCol, row))&
          srcData(row+1)%PN%sctsIsNull = .true.
        if (isNull(m1SctsCol, row) .or. isNull(m1OntimeCol, row))&
          srcData(row+1)%M1%sctsIsNull = .true.
        if (isNull(m2SctsCol, row) .or. isNull(m2OntimeCol, row))&
          srcData(row+1)%M2%sctsIsNull = .true.
      end do

      ! The call readColDataToFixed is in ssclib/dal_aux.
      call readColDataToFixed(pnSctsCol,          srcData%PN%scts)
      call readColDataToFixed(tab, 'PN_DET_ML',   srcData%PN%detml)
      call readColDataToFixed(tab, 'PN_MASKFRAC', srcData%PN%maskFrac)
      call readColDataToFixed(pnOntimeCol,   srcData%PN%ontime)

      call readColDataToFixed(m1SctsCol,          srcData%M1%scts)
      call readColDataToFixed(tab, 'M1_DET_ML',   srcData%M1%detml)
      call readColDataToFixed(tab, 'M1_MASKFRAC', srcData%M1%maskFrac)
      call readColDataToFixed(m1OntimeCol,   srcData%M1%ontime)

      call readColDataToFixed(m2SctsCol,          srcData%M2%scts)
      call readColDataToFixed(tab, 'M2_DET_ML',   srcData%M2%detml)
      call readColDataToFixed(tab, 'M2_MASKFRAC', srcData%M2%maskFrac)
      call readColDataToFixed(m2OntimeCol,   srcData%M2%ontime)

      if (hasColumn(tab, 'EP_1_ML_ID')) then
        call readColDataToFixed(tab, 'EP_1_ML_ID', srcData%mlIdSrc)
      else
        call warning('noEp1MlIdCol',"")
      end if

      call release(m2OntimeCol)
      call release(m1OntimeCol)
      call release(pnOntimeCol)
      call release(m2SctsCol)
      call release(m1SctsCol)
      call release(pnSctsCol)
    end if ! numSources > 0

    call release(tab)
    call release(set)

  end subroutine readSourceListData

  !---------------------------------------------------------------------
  subroutine initializeSrcDataType(srcData)
  !---------------------------------------------------------------------

    implicit none

    type(srcDataType), intent(inout) :: srcData(:)

    !-------------------------------------------------------------------
    ! Start

    srcData%scts       = 0.0
    srcData%binWidth   = 0.0
    srcData%sctsIsNull = .false.
    srcData%srcNumHex  = ''
    srcData%bitFlags   = 0
    srcData%srcNum     = 0
    srcData%mlIdSrc    = 0
    srcData%ra         = 0.0d0
    srcData%dec        = 0.0d0
    srcData%raCorr     = 0.0d0
    srcData%decCorr    = 0.0d0

    call initializeInstSrcDataType(srcData%PN)
    call initializeInstSrcDataType(srcData%M1)
    call initializeInstSrcDataType(srcData%M2)

  end subroutine initializeSrcDataType

  !---------------------------------------------------------------------
  subroutine initializeInstSrcDataType(instSrcData)
  !---------------------------------------------------------------------

    implicit none

    type(instSrcDataType), intent(inout) :: instSrcData(:)

    !-------------------------------------------------------------------
    ! Start

    instSrcData%sctsIsNull = .false.
    instSrcData%scts       = 0.0
    instSrcData%detml      = 0.0
    instSrcData%maskFrac   = 0.0
    instSrcData%ontime     = 0.0d0

  end subroutine initializeInstSrcDataType

  !---------------------------------------------------------------------
  subroutine getStartAndStopTimes(startTime, stopTime)
  !---------------------------------------------------------------------
  ! Reads TSTART and TSTOP kwds from all event list headers and computes the OR of these periods. This ORed interval gets written as TLMIN and TLMAX kwds to the time column of each event list and is also written to the output src list.

    implicit none

    real(double), intent(out) :: startTime, stopTime

    integer :: numEventLists, i
    real(double), allocatable :: startTimes(:), stopTimes(:)
    type(DataSetT) :: set
    type(TableT) :: tab
    type(ColumnT) :: col

    !-------------------------------------------------------------------
    ! Start

    numEventLists = size(eventSetNames)
    if (numEventLists <= 0)&
call error('bug1',"This shouldn't happen. Please &
&contact the code developer.")

    allocate(startTimes(numEventLists), stopTimes(numEventLists))
    startTimes = 0.0d0
    stopTimes  = 0.0d0
    do i = 1, numEventLists
      set = dataSet(eventSetNames(1), READ)
      tab = table(set, 'EVENTS')
      startTimes(i) = real64Attribute(tab, 'TSTART')
      stopTimes( i) = real64Attribute(tab, 'TSTOP')
      call release(tab)
      call release(set)
    end do

    startTime = minval(startTimes)
    stopTime  = maxval(stopTimes)
if (startTime >= stopTime) call error('bug2',"Please contact the code developer.")

    do i = 1, numEventLists
      set = dataSet(eventSetNames(1), MODIFY)
      tab = table(set, 'EVENTS')
      col = column(tab, 'TIME')
      call setAttribute(col, 'TLMIN', startTime)
      call setAttribute(col, 'TLMAX', stopTime)
      call release(col)
      call release(tab)
      call release(set)
    end do

  end subroutine getStartAndStopTimes

  !---------------------------------------------------------------------
  subroutine writeSourceListData(srcData, startTime, stopTime)
  !---------------------------------------------------------------------

    implicit none

    type(srcDataType), intent(in) :: srcData(:)
    real(double),      intent(in) :: startTime, stopTime

    integer :: numSources, numLiveSources, row, i
    type(DataSetT) :: set
    type(TableT) :: tab
    type(ColumnT) ::&
      srcNumCol, raCol, decCol, raCorrCol, decCorrCol, binWidthCol, srcNumHexCol,&
      bitFlagsCol, epicSctsCol, pnSctsCol, m1SctsCol, m2SctsCol, mlIdSrcCol
    integer(int32), pointer :: srcNumData(:), mlIdSrcData(:)
    real(double), pointer ::&
      raData(:), decData(:), raCorrData(:), decCorrData(:), binWidthData(:)
    integer(int8), pointer :: bitFlagsData(:)
    real(single), pointer :: epicSctsData(:), pnSctsData(:), m1SctsData(:), m2SctsData(:)

    !-------------------------------------------------------------------
    ! Start

    numSources = size(srcData)
    numLiveSources = count(srcData%bitFlags > 0)

    set = dataSet(outputSourceSetName, CREATE)
    tab = addTable(set, 'SRCLIST', numLiveSources)

    call setAttribute(tab, 'PN_P_BIT', pnFlagBit)
    call setAttribute(tab, 'M1_P_BIT', m1FlagBit)
    call setAttribute(tab, 'M2_P_BIT', m2FlagBit)

    call setAttribute(tab, 'TSTART', startTime)
    call setAttribute(tab, 'TSTOP',  stopTime)

    if (numLiveSources > 0) then
      srcNumCol    = addColumn(tab, 'SRC_NUM',     INTEGER32)
      raCol        = addColumn(tab, 'RA',          REAL64,    'deg')
      decCol       = addColumn(tab, 'DEC',         REAL64,    'deg')
      raCorrCol    = addColumn(tab, 'RA_CORR',     REAL64,    'deg')
      decCorrCol   = addColumn(tab, 'DEC_CORR',    REAL64,    'deg')
      binWidthCol  = addColumn(tab, 'TBIN_WIDTH',  REAL64,    's')
      srcNumHexCol = addColumn(tab, 'SRC_NUM_HEX', STRING&
        , dimensions=(/maxNumHexSrcChars/))
      bitFlagsCol  = addColumn(tab, 'PROCESS',     INTEGER8&
        , comment='If the relevant bit is set, make products for this src.')
      epicSctsCol  = addColumn(tab, 'EP_CTS',      REAL32,    'counts'&
        , comment='Equivalent epic source counts')
      pnSctsCol    = addColumn(tab, 'PN_CTS',      REAL32,    'counts')
      m1SctsCol    = addColumn(tab, 'M1_CTS',      REAL32,    'counts')
      m2SctsCol    = addColumn(tab, 'M2_CTS',      REAL32,    'counts')
      mlIdSrcCol   = addColumn(tab, 'ML_ID_SRC',   INTEGER32)

      row = 0
      do i = 1, numSources
      if (srcData(i)%bitFlags == 0) cycle

        if (srcData(i)%sctsIsNull)&
          call setToNull(epicSctsCol, row)
        if (srcData(i)%PN%sctsIsNull)&
          call setToNull(pnSctsCol, row)
        if (srcData(i)%M1%sctsIsNull)&
          call setToNull(m1SctsCol, row)
        if (srcData(i)%M2%sctsIsNull)&
          call setToNull(m2SctsCol, row)
        if (srcData(i)%mlIdSrc <= 0)&
          call setToNull(mlIdSrcCol, row)

        row = row + 1
      end do

      srcNumData    => int32Data(   srcNumCol)
      raData        => real64Data(      raCol)
      decData       => real64Data(     decCol)
      raCorrData    => real64Data(  raCorrCol)
      decCorrData   => real64Data( decCorrCol)
      binWidthData  => real64Data(binWidthCol)
      bitFlagsData  => int8Data(  bitFlagsCol)
      epicSctsData  => real32Data(epicSctsCol)
      pnSctsData    => real32Data(  pnSctsCol)
      m1SctsData    => real32Data(  m1SctsCol)
      m2SctsData    => real32Data(  m2SctsCol)
      mlIdSrcData   => int32Data(  mlIdSrcCol)

      row = 0
      do i = 1, numSources
      if (srcData(i)%bitFlags == 0) cycle

        srcNumData(  row) = srcData(i)%srcNum
        raData(      row) = srcData(i)%ra
        decData(     row) = srcData(i)%dec
        raCorrData(  row) = srcData(i)%raCorr
        decCorrData( row) = srcData(i)%decCorr
        binWidthData(row) = srcData(i)%binWidth
        bitFlagsData(row) = srcData(i)%bitFlags

!        call setStringCell(srcNumHexCol, row, trim(srcData(i)%srcNumHex))
        call setStringCell(srcNumHexCol, row, srcData(i)%srcNumHex)
!*** check that this truncates ok

        if (.not. srcData(i)%sctsIsNull)&
          epicSctsData(row) = srcData(i)%scts
        if (.not. srcData(i)%PN%sctsIsNull)&
          pnSctsData(row) = srcData(i)%PN%scts
        if (.not. srcData(i)%M1%sctsIsNull)&
          m1SctsData(row) = srcData(i)%M1%scts
        if (.not. srcData(i)%M2%sctsIsNull)&
          m2SctsData(row) = srcData(i)%M2%scts
        if (srcData(i)%mlIdSrc > 0)&
          mlIdSrcData(row) = srcData(i)%mlIdSrc

        row = row + 1
      end do

      call release(mlIdSrcCol)
      call release(m2SctsCol)
      call release(m1SctsCol)
      call release(pnSctsCol)
      call release(epicSctsCol)
      call release(bitFlagsCol)
      call release(srcNumHexCol)
      call release(binWidthCol)
      call release(decCol)
      call release(raCol)
      call release(srcNumCol)
    end if

    call release(tab)
    call release(set)

  end subroutine writeSourceListData

  !---------------------------------------------------------------------
  subroutine setFlags(srcIsSelected)
  !---------------------------------------------------------------------

    implicit none

    logical, intent(in) :: srcIsSelected(:)

    type(DataSetT) :: set
    type(TableT) :: tab
    type(ColumnT) :: col0, col1
    logical(bool), pointer :: spectraFlag(:), tSeriesFlag(:)

    !-------------------------------------------------------------------
    ! Start

    set = dataSet(srcmatchSourceSetName, MODIFY)
    tab = table(set, srcmatchSourceTabName)
    col0 = column(tab, 'SPECTRA')
    col1 = column(tab, 'TSERIES')
    spectraFlag => boolData(col0)
    tSeriesFlag => boolData(col1)

    spectraFlag = .false.
    tSeriesFlag = .false.
    where(srcIsSelected)
      spectraFlag = .true.
      tSeriesFlag = .true.
    end where

    call release(col1)
    call release(col0)
    call release(tab)
    call release(set)

  end subroutine setFlags

  !---------------------------------------------------------------------
  function intToHex(intArg) result(hexStr)
  !---------------------------------------------------------------------
  ! Converts the integer into a left-justified hexadecimal string.
!********* put it into ssclib eventually

!**** can't this be done with write(hexStr, '(z8)') intArg??

    implicit none

    integer(int32), intent(in) :: intArg
    integer, parameter :: numHexChars = 7
    character(numHexChars)     :: hexStr ! int32 integer can't give a
                                         ! longer hex string.

    character(1) :: additionChar
    integer :: oldMantissaInt, numChars, newMantissaInt, remainderInt

    !-------------------------------------------------------------------
    ! Start

    hexStr = repeat(' ', numHexChars)

if (intArg < 0) call error('negativeIntArg',"")
if (log(real(intArg)) >= 8.0*log(16.0)) call error('intArgTooBig',"")

    if (intArg == 0) then
      hexStr = '0'//repeat(' ', numHexChars-1)
  return
    end if

    oldMantissaInt = intArg
    numChars = 0
    do while(oldMantissaInt > 0)
      newMantissaInt = floor(oldMantissaInt / 16.0)
      remainderInt = oldMantissaInt - 16 * newMantissaInt
      numChars = numChars + 1
if (numChars > numHexChars) call error('bug2',"Please contact the code developer.")

      select case(remainderInt)
      case(10)
        hexStr = 'A'//trim(hexStr)
      case(11)
        hexStr = 'B'//trim(hexStr)
      case(12)
        hexStr = 'C'//trim(hexStr)
      case(13)
        hexStr = 'D'//trim(hexStr)
      case(14)
        hexStr = 'E'//trim(hexStr)
      case(15)
        hexStr = 'F'//trim(hexStr)
      case default
        write(additionChar, '(i1)') remainderInt
        hexStr = additionChar//trim(hexStr)
      end select

      oldMantissaInt = newMantissaInt
    end do

    hexStr = hexStr//repeat(' ', numHexChars-len_trim(hexStr))

  end function intToHex

  !---------------------------------------------------------------------
  function intToHex_20051028(intArg) result(hexStr)
  !---------------------------------------------------------------------
  ! Converts the integer into a left-justified hexadecimal string.
!********* put it into ssclib eventually

    implicit none

    integer(int32), intent(in) :: intArg
    character(7)               :: hexStr ! int32 integer can't give a
                                         ! longer hex string.

    character(1) :: additionChar
    integer :: oldMantissaInt, numChars, newMantissaInt, remainderInt

    !-------------------------------------------------------------------
    ! Start

    hexStr = ''

if (intArg < 0) call error('negativeIntArg',"")
if (log(real(intArg)) >= 8.0*log(16.0)) call error('intArgTooBig',"")

    if (intArg == 0) then
      hexStr = '0'
  return
    end if

    oldMantissaInt = intArg
    numChars = 0
    do while(oldMantissaInt > 0)
      newMantissaInt = floor(oldMantissaInt / 16.0)
      remainderInt = oldMantissaInt - 16 * newMantissaInt
      numChars = numChars + 1
if (numChars > 7) call error('bug3',"Please contact the code developer.")

      select case(remainderInt)
      case(10)
        hexStr = trim(hexStr)//'A'
      case(11)
        hexStr = trim(hexStr)//'B'
      case(12)
        hexStr = trim(hexStr)//'C'
      case(13)
        hexStr = trim(hexStr)//'D'
      case(14)
        hexStr = trim(hexStr)//'E'
      case(15)
        hexStr = trim(hexStr)//'F'
      case default
        write(additionChar, '(i1)') remainderInt
!        write(additionChar ('i1')) remainderInt
        hexStr = trim(hexStr)//additionChar
      end select

      oldMantissaInt = newMantissaInt
    end do

  end function intToHex_20051028

  !---------------------------------------------------------------------
  subroutine template
  !---------------------------------------------------------------------

    implicit none

    !-------------------------------------------------------------------
    ! Start

  end subroutine template

end module esources_mod
